/*
	rpipic - A PIC Programmer for the Raspberry Pi
	Copyright (C) 2012  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

#include "board.hpp"

Breadboard::Breadboard(Gpio &gpio, int vdd, int vpp, int clock, int data_out, int data_in) :
	m_gpio(gpio), m_vdd(vdd), m_vpp(vpp), m_clock(clock), m_data_out(data_out), m_data_in(data_in)
{
	m_gpio.Enable(m_vdd, true);
	m_gpio.Enable(m_vpp, true);
	m_gpio.Enable(m_clock, true);
	m_gpio.Enable(m_data_out, true);
	m_gpio.Enable(m_data_in, false);
	
	SetVddHigh(false);
	SetVppHigh(false);
	SetClockHigh(false);
	SetDataHigh(false);
}

Breadboard::~Breadboard()
{
	m_gpio.Disable(m_vdd);
	m_gpio.Disable(m_vpp);
	m_gpio.Disable(m_clock);
	m_gpio.Disable(m_data_out);
	m_gpio.Disable(m_data_in);
}

void Breadboard::SetVddHigh(bool high)
{
	m_gpio.Write(m_vdd, !high);
}

void Breadboard::SetVppHigh(bool high)
{
	m_gpio.Write(m_vpp, !high);
}

void Breadboard::SetClockHigh(bool high)
{
	m_gpio.Write(m_clock, !high);
}

void Breadboard::SetDataHigh(bool high)
{
	m_gpio.Write(m_data_out, !high);
}

bool Breadboard::GetDataHigh()
{
	return m_gpio.Read(m_data_in);
}

