/*
	rpipic - A PIC Programmer for the Raspberry Pi
	Copyright (C) 2012  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/


#include <ctime>

#include "chip.hpp"

Chip::Chip(Board &board) :
	m_board(board),
	m_read_ep(1000000),
	m_set(1000000),
	m_hold(1000000)
{
}

void Chip::InitChip(int64_t read_ep, int64_t set, int64_t hold)
{
	m_read_ep = read_ep;
	m_set = set;
	m_hold = hold;
}

Board &Chip::GetBoard()
{
	return m_board;
}

uint16_t Chip::Read(unsigned int n)
{
	uint16_t bits = 0;
	for (unsigned int i = 0; i < n; i++){
		m_board.SetClockHigh(true);
		Wait(m_read_ep);
		bits |= m_board.GetDataHigh() ? (1 << i) : 0;
		Wait(m_set);
		m_board.SetClockHigh(false);
		Wait(m_hold);
	}
	m_board.SetDataHigh(false);
	return bits;
}

void Chip::Write(uint16_t bits, unsigned int n)
{
	for (unsigned int i = 0; i < n; i++){
		m_board.SetClockHigh(true);
		m_board.SetDataHigh(bits & (1 << i));
		Wait(m_set);
		m_board.SetClockHigh(false);
		Wait(m_hold);
	}
	m_board.SetDataHigh(false);
}

void Chip::Wait(int64_t ns)
{
	timespec t;
	timespec r;
	
	t.tv_sec = ns/(int64_t)1000000000;
	t.tv_nsec = ns % (int64_t)1000000000;
	r.tv_sec = 0;
	r.tv_nsec = 0;
	
	while (nanosleep (&t, &r));
}
