/*
	rpipic - A PIC Programmer for the Raspberry Pi
	Copyright (C) 2012  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/


#ifndef CHIP_HPP
#define CHIP_HPP

#include <stdint.h>

#include "board.hpp"
#include "data.hpp"

class Chip
{
	public:
		Chip(Board &board);
		
		virtual void PrintMeta(Data &data) = 0;
		
		virtual void ReadProgram(Data &data) = 0;
		virtual void ReadData(Data &data) = 0;
		virtual void ReadConfiguration(Data &data) = 0;
		virtual void ReadUserId(Data &data) = 0;
		virtual void ReadDeviceId(Data &data) = 0;
		
		virtual void WriteProgram(Data &data) = 0;
		virtual void WriteData(Data &data) = 0;
		virtual void WriteConfiguration(Data &data) = 0;
		virtual void WriteUserId(Data &data) = 0;
		
		virtual void Erase() = 0;
		
		static void Wait(int64_t ns);
		
	protected:
		static const char *FormatBits(uint32_t bits, unsigned int n);
		void InitChip(int64_t read_ep, int64_t set, int64_t hold);
		Board &GetBoard();
		
		uint16_t Read(unsigned int n);
		void Write(uint16_t bits, unsigned int n);
		
	private:
		Board &m_board;
		int64_t m_read_ep;
		int64_t m_set;
		int64_t m_hold;
};

#endif

