/*
	rpipic - A PIC Programmer for the Raspberry Pi
	Copyright (C) 2012  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/


#ifndef DATA_HPP
#define DATA_HPP

#include <cstdio>
#include <stdint.h>
#include <list>
#include <vector>

class Data
{
	public:
		uint8_t GetData(uint32_t address);
		virtual bool GetData(uint32_t address, uint8_t &word) = 0;
		virtual void SetData(uint32_t address, uint8_t word) = 0;
};


struct HexDataItem
{
	uint32_t address;
	std::vector<uint8_t> bytes;
};

class Hex : public Data
{
	public:
		virtual bool GetData(uint32_t address, uint8_t &byte);
		virtual void SetData(uint32_t address, uint8_t byte);
		void ReadFile(FILE *f);
		void WriteFile(FILE *f);
		
	private:
		uint8_t ConvertHex(const char a, const char b);
		
		std::list<HexDataItem> items;
};

#endif

