/*
	rpipic - A PIC Programmer for the Raspberry Pi
	Copyright (C) 2012  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/


#ifndef PIC_HPP
#define PIC_HPP

#include <stdint.h>
#include "chip.hpp"

class Pic : public Chip
{
	public:
		Pic(Board &board);
		virtual ~Pic();
		void InitPic(int64_t vpptime, int64_t init_hold, int64_t cmd_delay, int64_t erase_time, int64_t end_program_time, int64_t program_time);
		
	protected:
		void Reset();
		void IncrementAddress();
		void LoadConfiguration();
		void BeginEraseProgramCycle();
		void EndEraseProgramCycle();
		
		uint16_t ReadProgramWord(unsigned int bits);
		uint8_t ReadDataWord(unsigned int bits);
		void WriteProgramWord(uint16_t word, unsigned int bits);
		void WriteDataWord(uint16_t word, unsigned int bits);
		
		void PicReadProgram(Data &data, uint16_t pic_address, uint16_t data_address, uint16_t length,
			unsigned int bits, bool configuration = false, bool increment_address = false);
		void PicReadData(Data &data, uint16_t pic_address, uint16_t data_address, uint16_t length);
		void PicWriteProgram(Data &data, uint16_t pic_address, uint16_t data_address, uint16_t length,
			unsigned int bits,  bool end_program, bool configuration = false, bool increment_address = false);
		void PicWriteData(Data &data, uint16_t pic_address, uint16_t data_address, uint16_t length,
			bool end_program);
		
	private:
		int64_t m_vpptime;
		int64_t m_init_hold;
		int64_t m_cmd_delay;
		int64_t m_erase_time;
		int64_t m_end_program_time;
		int64_t m_program_time;
};

#endif

