#include p16f5x.inc

; ======== CONFIGURATION ======== ;
	LIST p=16f54
	__CONFIG _CP_OFF & _WDT_OFF & _XT_OSC ; _RC_OSC
	__IDLOCS 0x1234

; ======== VARIABLES ======== ;
WAIT1k_1	equ	0x07
WAIT1M_1	equ	0x08

; ======== MAIN PROGRAM ======== ;
; ==== START ==== ;
start:
	; Initialise and configure Port B
	clrf PORTB ; Set all of port B to 0
	movlw 0xFC ; Output on PortB 0 and 1
	tris PORTB ; Configure
	
; ==== LOOP FOREVER ==== ;
; This loop actually takes 1000010 cycles to complete:p
loop:
	movlw 0x03 ; RB0 and RB1 at 1
	movwf PORTB ; Write to port B
	call wait1M
	
	movlw 0x00
	movwf PORTB
	call wait1M
	
	movlw 0x01
	movwf PORTB
	call wait1M
	
	movlw 0x02
	movwf PORTB
	call wait1M
	
	goto loop

; ======== Procedures ======== ;
; ==== Wait 1000000 cycles ==== ;
; 2 cycles being called
; 2000 cycles before setting up the main loop
; 2 cycles setting up the main loop
; 3 cycles of nops
; 997991 (4008 * 248 + 4007) cycles in the second loop (leaving 2009 cycles)
; 2 cycles to return
wait1M:
	; use the spare 2000 cycles
	call wait1k
	call wait1k
	
	; set up for the loop
	movlw D'249' ; See timing above
	movwf WAIT1M_1 ; Set the variable to this
	
	nop
	nop
	nop
	
	; This loop body consumes 4008 cycles on every iteration,
	; Except the lasts, on which it consumes 4007
wait1M_loop:
		call wait1k
		call wait1k
		call wait1k
		call wait1k
		nop
		nop
		nop
		nop
		nop
		decfsz WAIT1M_1 ; Decrement the counter; skip if zero
		goto wait1M_loop ; Loop otherwise
	
	retlw 0 ; Return from the procedure once we get here

; ==== Waits 1000 cycles ==== ;
; 2 Cycles being called
; 2 cycles before the nops
; 3 cycles of nops
; 991 cycles in the loop (988 + 3)
; 2 cycles returning
wait1k:
	movlw D'248' ; Let's wait 991 (247 * 4 + 3) cycles in the loop
	movwf WAIT1k_1 ; Set the variable to this
	
	; Make this procedure a multiple of 4 cycles (including call)
	nop
	nop
	nop
	
	; This loop body consumes 4 cycles on every iteration,
	; Except the lasts, on which it consumes 3
wait1k_loop: ; Loop until we're done
		nop ; make it take the required number of cycles
		decfsz WAIT1k_1 ; Decrement the counter; skip if zero
		goto wait1k_loop ; Loop otherwise
	
	retlw 0 ; Return from the procedure once we get here

; ======== END ======== ;
end
